import argparse
import subprocess
from subprocess import CalledProcessError
from pathlib import Path


def get_args():
    parser = argparse.ArgumentParser()
    parser.add_argument('indir', type=Path, help='Directory containing artist directories')
    parser.add_argument('outdir', type=Path, help='Empty directory where transcodes will be placed')
    parser.add_argument('encoder', type=Path, help='Location of encoder')
    return parser.parse_args()


def skip_all_albums(path: Path):
    for directory in path.iterdir():
        if directory.is_dir():
            if not skip_album(directory):
                return False
        else:
            print(f"Warning, skipping non-dir '{directory}' found in '{path}'")
    return True


def skip_album(path: Path):
    for file in path.iterdir():
        if file.name == "DONE":
            return True
    return False


def transcode(transcode_list: list, encoder: Path):
    additional_args = ('--bitrate', '128', '--music')
    for in_track, out_track in transcode_list:
        subprocess_args = (str(encoder),) + additional_args + (in_track, out_track)
        try:
            result = subprocess.run(subprocess_args, capture_output=True, text=True, check=True)
        except CalledProcessError:
            print(f"ERROR: Transcoding of '{in_track}' failed with errors:")
            print(result.stderr)


def main(input_dir: Path, output_dir: Path, encoder: Path):
    transcode_list = []
    for artist in input_dir.iterdir():
        if artist.is_dir():
            artist_out = Path(output_dir) / artist.name
            if skip_all_albums(artist):
                print(f"Skipping '{artist.parts[-1]}'")
                continue
            else:
                artist_out.mkdir()
                for album in artist.iterdir():
                    if album.is_dir():
                        album_out = artist_out / album.parts[-1]
                        if skip_album(album):
                            print(f"Skipping '{artist.parts[-1]} / {album.parts[-1]}'")
                        else:
                            album_out.mkdir()
                            # todo: copy files not on blacklist
                            # todo: create blacklist
                            for track in album.iterdir():
                                if track.is_file() and track.suffix.lower() == '.flac':
                                    transcode_list.append((str(track), str(album_out / f"{track.stem}.opus")))
                    else:
                        print(f"Warning, skipping non-dir '{album}' found in artist '{artist.parts[-1]}'")
                        continue
        else:
            print(f"Warning, skipping non-dir '{artist}' found in root")
            continue
    transcode(transcode_list, encoder)


if __name__ == '__main__':
    args = get_args()
    main(args.indir, args.outdir, args.encoder)
